#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *
from w3bsite.classes import utils, views
from w3bsite.classes import defaults as _defaults_

# the payments views.
class Views(_defaults_.Defaults):
	def __init__(self, 
		# passed Website.x objects.
		defaults=None,
	):
		# defaults.
		_defaults_.Defaults.__init__(self)
		self.assign(defaults.dict())
		
		# urlpatterns.
		if dev0s.system.env.get("MIGRATIONS", format=bool, default=False):
			self.urls = []
		else:
			self.urls = views.build_urls([
				self.Purchase(defaults=defaults),
				self.Cancel(defaults=defaults),
			])

	# purchase.
	class Purchase(views.View):
		def __init__(self, defaults=None):
			views.View.__init__(self, "payments/", "purchase", 
				website=defaults.website, 
				html=f"w3bsite/classes/apps/payments/html/purchase.html",
				auth_required=False,)
			_defaults_.Defaults.__init__(self)
			self.assign(defaults.dict())
		def view(self, request):
			user = {}
			response = self.website.users.authenticated(request)
			if response.success: 
				user = {"email":response.email, "username":response.username, "api_key":response.api_key}
			return self.render(request, {
				"USER":user,
			})

	# cancel.
	class Cancel(views.View):
		def __init__(self, defaults=None):
			views.View.__init__(self, "payments/", "cancel", 
				website=defaults.website, 
				html=f"w3bsite/classes/apps/payments/html/cancel.html",
				auth_required=False,)
			_defaults_.Defaults.__init__(self)
			self.assign(defaults.dict())
		def view(self, request):
			user = {}
			response = self.website.users.authenticated(request)
			if response.success: 
				user = {"email":response.email, "username":response.username, "api_key":response.api_key}
			return self.render(request, {
				"USER":user,
			})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *
from w3bsite.classes.utils import utils
import shutil, math, time

# the system control class.
class Logging(Object):
	def __init__(self, attributes={}):

		# docs.
		DOCS = {
			"module":"website.logging", 
			"initialized":True,
			"description":[], 
			"chapter": "Logging", }

		# defaults.
		Object.__init__(self)
		self.assign(attributes)
		
		# attributes.
		if isinstance(self.database, (str,String,FilePath)):
			self.database = Directory(str(self.database))

		# objects.
		self.alerts = self.Alerts(attributes=self.dict())

		#

	# copied functions.
	def log(self, 
		# option 1:
		# the message (#1 param).
		message=None,
		# option 2:
		# the error.
		error=None,
		# option 3:
		# the response dict (leave message None to use).
		response={},
		# print the response as json.
		json=False,
		# optionals:
		# the active log level.
		log_level=0,
		# the required log level for when printed to console (leave None to use self.log_level).
		required_log_level=None, 
		# save to log file.
		save=False,
		# save errors always (for options 2 & 3 only).
		save_errors=None,
		# the log mode (leave None for default).
		mode=None,
	):
		return dev0s.response.log(
			message=message,
			error=error,
			response=response,
			json=json,
			log_level=log_level,
			required_log_level=required_log_level,
			save=save,
			save_errors=save_errors,
			mode=mode,)
	def load_logs(self, format="webserver", options=["webserver", "cli", "array", "string"]):
		return dev0s.response.load_logs(format=format)
		#
	def reset_logs(self, format="webserver", options=["webserver", "cli", "array", "string"]):
		return dev0s.response.reset_logs(format=format)
		#
		
		#
	def log_to_file(self, message, raw=False):
		return dev0s.response.log_to_file(message, raw=raw)
		#

	# the alerts object class.
	class Alerts(Object):
		def __init__(self, attributes={}):

			# docs.
			DOCS = {
				"module":"website.logging.alerts", 
				"initialized":True,
				"description":[], 
				"chapter": "Logging", }

			# defaults.
			Object.__init__(self)
			self.assign(attributes)

			# attributes.
			if self.live:
				self.alerts = Dictionary({}, path=self.database.join("logs/alerts"), default={})
			else:
				self.alerts = Dictionary({}, path=self.database.join("logs/alerts"))

			#

		# save an alert.
		def save(self, 
			# the alert's id (str).
			id="testalert",
			# the alert's title (str).
			title="Warning!", 
			# the alert's message (str). 
			message="Some message.",
			# the alert's right button redirect url (str).  
			redirect="/dashboard/home/", 
			# the alert's right button redirect text (str).  
			redirect_button="Ok", 
			# the alert's icon path.  
			icon="/media/icons/warning.png",
			# the urls on which the alert will be shown (list) (use [*] for all urls).
			urls=["*"],
			# the users to which the alert will be shown (list) (use [*] for all users).
			users=["*"],
		):
			try: alerts = self.alerts.load()
			except: alerts = {}
			try: alerts[id]
			except: alerts[id] = {}
			if Files.exists(self.database.join("logs/alerts")):
				size = gfp.size(path=self.database.join("logs/alerts"), mode="mb", format="integer")
				if size >= 25: 
					alerts = {id:{}}
			alerts[id][Date().seconds_timestamp] = {
				"title":title,
				"message":message,
				"redirect":redirect,
				"redirect_button":redirect_button,
				"icon":icon,
				"timestamp":Date().seconds_timestamp,
				"urls":urls,
				"users":users,
				"read":False,
			}
			self.alerts.dictionary = alerts
			self.alerts.save()
			#

		# check alerts.
		def check(self,
			# specific alert id's (str, list) (optional).
			id=None,
			# the active user (str) (optional).
			username=None,
			# the active url (str) (optional).
			url=None,
		):	
			# checks.
			if isinstance(id, (str, String)):
				id = [id]

			# iterate.
			for _id_, alerts in self.alerts.items():
				for timestamp, _alert_ in alerts.items():
					if not alert["read"]:
						if (
							(id == None or _id_ in id) 
							and 
							(username == None or username in alert["users"]) 
							and 
							(url == None or url in alert["urls"])
						):
							return dev0s.response.success("Successfully checked the alerts.", {
								"alert":alert,
							})
			return dev0s.response.success("Successfully checked the alerts.", {
				"alert":None,
			})


		# mark a alert as read.
		def mark(self, 
			# the alert's id.
			id=None, 
			# the alert's timestamp.
			timestamp=None,
		):
			self.alerts.load()
			try:
				self.alerts[id][timestamp]['read'] = True
			except: 
				self.logging.log(f"Failed to mark alert {id} {timestamp}, alert [{id}] does not exist.")
				return None
			self.alerts.save()
			#

		#
	
	#	
#	
	
#


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *
from w3bsite.classes import utils, views
from w3bsite.classes import defaults as _defaults_

# app.
APP = "authentication"

# the x views.
class Views(_defaults_.Defaults):
	def __init__(self, 
		# passed Website.x objects.
		defaults=None,
	):
		# defaults.
		_defaults_.Defaults.__init__(self)
		self.assign(defaults.dict())
		
		# urlpatterns.
		if dev0s.system.env.get("MIGRATIONS", format=bool, default=False):
			self.urls = []
		else:
			self.urls = views.build_urls([
				self.SignIn(defaults=defaults),
				self.SignUp(defaults=defaults),
				self.Reset(defaults=defaults),
				self.Activate(defaults=defaults),
			])

	# sign in.
	class SignIn(views.View):
		def __init__(self, defaults=None):
			_defaults_.Defaults.__init__(self)
			views.View.__init__(self, f"{APP}/", "signin", html=f"w3bsite/classes/apps/authentication/html/signin.html", website=defaults.website)
			self.assign(defaults.dict())
		def view(self, request):
			if self._maintenance_: return self.maintenance(request)
			return self.render(request)

	# sign up.
	class SignUp(views.View):
		def __init__(self, defaults=None):
			_defaults_.Defaults.__init__(self)
			views.View.__init__(self, f"{APP}/", "signup", html=f"w3bsite/classes/apps/authentication/html/signup.html", website=defaults.website)
			self.assign(defaults.dict())
		def view(self, request):
			if self._maintenance_: return self.maintenance(request)
			return self.render(request)

	# reset password.
	class Reset(views.View):
		def __init__(self, defaults=None):
			_defaults_.Defaults.__init__(self)
			views.View.__init__(self, f"{APP}/", "reset", html=f"w3bsite/classes/apps/authentication/html/reset.html", website=defaults.website)
			self.assign(defaults.dict())
		def view(self, request):
			if self._maintenance_: return self.maintenance(request)
			return self.render(request)

	# activate account.
	class Activate(views.View):
		def __init__(self, defaults=None):
			_defaults_.Defaults.__init__(self)
			views.View.__init__(self, f"{APP}/", "activate", html=f"w3bsite/classes/apps/authentication/html/activate.html", website=defaults.website)
			self.assign(defaults.dict())
		def view(self, request):
			if self._maintenance_: return self.maintenance(request)
			return self.render(request)

	



# classes imports.
from w3bsite.classes.config import *
from w3bsite.classes import utils, views
from w3bsite.classes import defaults as _defaults_

# the authentication requests.
class Requests(_defaults_.Defaults):
	def __init__(self, 
		# passed Website.x objects.
		defaults=None,
	):
		# defaults.
		_defaults_.Defaults.__init__(self)
		self.assign(defaults.dict())

		# urlpatterns.
		if dev0s.system.env.get("MIGRATIONS", format=bool, default=False):
			self.urls = []
		else:
			self.urls = views.build_urls([
				self.Requests(defaults=defaults),
				self.AuthRequests(defaults=defaults),
				self.DiskSpace(defaults=defaults),
			])

	# get requests metrics.
	class Requests(views.Request):
		def __init__(self, defaults=None):
			views.Request.__init__(self, "requests/metrics/", "requests", website=defaults.website)
			self.assign(defaults.dict())
		def request(self, request):

			# check signed in.
			if dev0s.system.env.get("METRICS_AUTH_REQUIRED", format=bool, default=True):
				response = self.website.users.authenticated(request)
				if not response.success: return response
				elif request.user.username not in ["administrator"]:
					return self.permission_denied(request)

			# make request.
			return self.response(self.website.metrics.requests())

			#

	# get auth requests metrics.
	class AuthRequests(views.Request):
		def __init__(self, defaults=None):
			views.Request.__init__(self, "requests/metrics/", "auth_requests", website=defaults.website)
			self.assign(defaults.dict())
		def request(self, request):

			# check signed in.
			if dev0s.system.env.get("METRICS_AUTH_REQUIRED", format=bool, default=True):
				response = self.website.users.authenticated(request)
				if not response.success: return response
				elif request.user.username not in ["administrator"]:
					return self.permission_denied(request)

			# make request.
			return self.response(self.website.metrics.auth_requests())
			
			#

	# get disk space.
	class DiskSpace(views.Request):
		def __init__(self, defaults=None):
			views.Request.__init__(self, "requests/metrics/", "disk_space", website=defaults.website)
			self.assign(defaults.dict())
		def request(self, request):

			# check signed in.
			if dev0s.system.env.get("METRICS_AUTH_REQUIRED", format=bool, default=True):
				response = self.website.users.authenticated(request)
				if not response.success: return response
				elif request.user.username not in ["administrator"]:
					return self.permission_denied(request)

			# make request.
			return self.response(self.website.metrics.disk_space())
		
		#

#


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from w3bsite.classes.config import *
from w3bsite.classes import exceptions

# the database class.
class Database(Traceback):
	def __init__(self, firestore=None, path=None, live=False):
		
		# docs.
		DOCS = {
			"module":"website.db", 
			"initialized":True,
			"description":[], 
			"chapter": "Database", }

		# traceback.
		Traceback.__init__(self, traceback="w3bsite.Website.db")

		# checks.
		if firestore == None and path == None: raise exceptions.InvalidUsage(self.__traceback__()+" Both parameters firestore & path are None.")

		# attributes.
		self.mode = "firestore"
		self.live = live
		self.firestore = firestore
		self.path = gfp.clean(path)
		self.file_path = self.fp = FilePath(self.path)
		if firestore == None:
			self.database = dev0s.database.Database(self.path)
			self.mode = "cache"
			if self.live and not self.fp.exists():
				dev0s.response.log(f"&ORANGE&Root permission&END& required to create database [{self.path}].")
				os.system(f"sudo mkdir {self.path}")
				Files.chmod(path=self.path, permission=700, sudo=True)
				Files.chown(path=self.path, owner=dev0s.defaults.vars.user, group=dev0s.defaults.vars.group, sudo=True)

		#

	# functions.
	def load(self, path=None):
		if path == None: return dev0s.response.error(self.__traceback__(function="load")+" Define parameter: [path].")
		path = gfp.clean(path)
		if self.mode == "firestore":
			return self.firestore.load(path)
		elif self.mode == "cache":
			return self.database.load(path=path)
		else: raise exceptions.InvalidUsage(self.__traceback__(function="load")+f" Unknown mode [{self.mode}].")
	def save(self, path=None, data=None, overwrite=False):
		if path == None: return dev0s.response.error(self.__traceback__(function="save")+" Define parameter: [path].")
		if data == None: return dev0s.response.error(self.__traceback__(function="save")+" Define parameter: [data].")
		path = gfp.clean(path)
		if self.mode == "firestore":
			if isinstance(data, (dict, Dictionary)):
				response = self.load(path=path)
				if response.success:
					return self.firestore.save(path, Dictionary(response.data).insert(data))
				else:
					return self.firestore.save(path, data)
			else:
				return self.firestore.save(path, data)
		elif self.mode == "cache":
			return self.database.save(path=path, data=data, overwrite=overwrite)
		else: raise exceptions.InvalidUsage(self.__traceback__(function="save")+f" Unknown mode [{self.mode}].")
	def delete(self, path=None):
		if path == None: return dev0s.response.error(self.__traceback__(function="delete")+" Define parameter: [path].")
		path = gfp.clean(path)
		if self.mode == "firestore":
			return self.firestore.delete(path)
		elif self.mode == "cache":
			return self.database.delete(path=path)
		else: raise exceptions.InvalidUsage(self.__traceback__(function="delete")+f" Unknown mode [{self.mode}].")

	# get names.
	def names(self,
		# the sub path (leave None to use the root path)
		path=None,
	):

		# checks.
		if path == None: return dev0s.response.error(self.__traceback__(function="names")+" Define parameter: [path].")
		path = gfp.clean(path)

		# vars.
		names = []

		# firestore.
		if self.mode == "firestore":
			response = self.firestore.load_collection(path)
			if not response.success: response.crash()
			names = response["collection"]

		# cache.
		elif self.mode == "cache":
			return self.database.names(path=path)

		# handler.
		return names

		#

	# path functions.
	def subpath(self, fullpath):
		return self.file_path.clean(path=fullpath.replace(self.path, ""), remove_double_slash=True)
	def fullpath(self, subpath):
		return self.file_path.clean(path=f"{self.path}/{subpath}", remove_double_slash=True)
	def join(self, name=None, type=""):
		return self.file_path.join(name, type)
		#
		
	# representation.
	def __str__(self):
		return str(self.fp.path)
	def __repr__(self):
		return str(self)

	#

	
			